#include "Mat2d.h"
#include "Vec.h"
#include "MVoperators.h"
#include <assert.h>

Mat2d operator+(double d, const Mat2d& M) {
   Mat2d tmp(M);
   return tmp += d;
}

Mat2d operator-(double d, const Mat2d& M) {
   Mat2d tmp(M);
   for(double *it = tmp.data; it != tmp.data + tmp.n * tmp.m; ++it)
      *it = d - *it;
   return tmp;
}

Mat2d operator*(double d, const Mat2d& M) {
   Mat2d tmp(M);
   return tmp *= d;
}

Mat2d operator/(double d, const Mat2d& M) {
   Mat2d tmp(M);
   for(double *it = tmp.data; it != tmp.data + tmp.n * tmp.m; ++it)
      *it = d / *it;
   return tmp;
}

Vec operator*(const Mat2d& M, const Vec& V) {
   assert(M.n == V.n);
   Vec tmp(M.m);
   const double* itM = M.data;
   std::vector<double>::const_iterator itV;
   std::vector<double>::iterator itTmp = tmp.data.begin();
   for(int i = 0; i < M.m; ++i) {
      *itTmp = 0;
      itV = V.data.begin();
      for(int j = 0; j < M.n; ++j) {
         *itTmp += *itM++ * *itV++;
      }
      ++itTmp;
   }
   
   return tmp;
}

Vec operator+(double d, const Vec& V) {
   Vec tmp(V);
   return tmp += d;
}

Vec operator-(double d, const Vec& V) {
   Vec tmp(V);
   std::vector<double>::iterator endV = tmp.data.end();
   for(std::vector<double>::iterator it = tmp.data.begin(); it != endV; ++it)
      *it = d - *it;
   return tmp;
}
   
Vec operator*(double d, const Vec& V) {
   Vec tmp(V);
   return tmp *= d;
}

Vec operator/(double d, const Vec& V) {
   Vec tmp(V);
   std::vector<double>::iterator endV = tmp.data.end();
   for(std::vector<double>::iterator it = tmp.data.begin(); it != endV; ++it)
      *it = d / *it;
   return tmp;
}
