#ifndef MAT2D_H
#define MAT2D_H

#include "Vec.h"

class Vec;

class Mat2d {
private:
   double *data;
   int m;
   int n;

public:
   Mat2d();
   Mat2d(int m, int n);
   Mat2d(const Mat2d& M);
   ~Mat2d();
   void resize(int m, int n);
   int getM() const;
   int getN() const;
   void getMN(int &m, int &n) const;
   Mat2d& fill(double value);
   const double& getElem(int i, int j) const;
   double & getElem(int i, int j);
   Mat2d& add(const Mat2d& M);
   bool equal(const Mat2d& M) const;
   
   // additional operators
   bool operator==(const Mat2d& M) const;
   bool operator!=(const Mat2d& M) const;
   Mat2d& operator=(const Mat2d& M);
   Mat2d& operator=(double d);

   // operators
   const double& operator()(int i, int j) const;
   double& operator()(int i, int j);
   Mat2d& operator+=(const Mat2d& M);
   Mat2d& operator+=(double d);
   Mat2d& operator-=(const Mat2d& M);
   Mat2d& operator-=(double d);
   // elementwise multiplication
   Mat2d& operator*=(const Mat2d& M);
   Mat2d& operator*=(double d);
   Mat2d& operator/=(const Mat2d& M);
   Mat2d& operator/=(double d);
   Mat2d operator+(const Mat2d& M) const;
   Mat2d operator+(double d) const;
   Mat2d operator-(const Mat2d& M) const;
   Mat2d operator-(double d) const;
   Mat2d operator*(const Mat2d& M) const;
   Mat2d operator*(double d) const;
   Mat2d operator/(const Mat2d& M) const;
   Mat2d operator/(double d) const;

   friend Mat2d operator-(double d, const Mat2d& M);
   friend Mat2d operator/(double d, const Mat2d& M);

   friend Vec operator*(const Mat2d& M, const Vec& V);
};

#endif
