#include "Vec.h"
#include <vector>
#include <assert.h>

Vec::Vec() :
   n(0) {

}

Vec::Vec(int n) :
   n(n), data(n) {
   
}
//   Vec(const Vec& V); // default copy-constructor sufficient
//   no destructor needed
void Vec::resize(int n) {
   data.resize(n);
}

int Vec::getN() const {
   return n;
}

Vec& Vec::fill(double value) {
   data.assign(n, value);
   return *this;
}

const double& Vec::getElem(int i) const {
   return data.at(i); // checked version
}

double& Vec::getElem(int i) {
   return data.at(i);
}

Vec& Vec::add(const Vec& V) {
   std::vector<double>::const_iterator itV(V.data.begin());
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it += *itV++;
   return *this;
}
   
bool Vec::equal(const Vec& V) const {
   if(n != V.n)
      return false;

   std::vector<double>::const_iterator itV(V.data.begin());
   for(std::vector<double>::const_iterator it = data.begin(); it != data.end(); ++it) {
      if(*it != *itV++) {
         return false;
      }
   }
   return true;
}

const double& Vec::operator()(int i) const {
   return getElem(i);
}


// additional operators
bool Vec::operator==(const Vec& V) const {
   return this->equal(V);
}

bool Vec::operator!=(const Vec& V) const {
   return !this->equal(V);
}

Vec& Vec::operator=(double d) {
   return this->fill(d);
}

// operators

double& Vec::operator()(int i) {
   return getElem(i);
}

Vec& Vec::operator+=(const Vec& V) {
   assert(n == V.n);
   std::vector<double>::const_iterator itV = V.data.begin();   
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it += *itV++;
   return *this;
}

Vec& Vec::operator+=(double d) {
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it += d;
   return *this;
}

Vec& Vec::operator-=(const Vec& V) {
   assert(n == V.n);
   std::vector<double>::const_iterator itV = V.data.begin();   
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it -= *itV++;
   return *this;
}

Vec& Vec::operator-=(double d) {
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it -= d;
   return *this;
}

// elementwise multiplication
Vec& Vec::operator*=(const Vec& V) {
   assert(n == V.n);
   std::vector<double>::const_iterator itV = V.data.begin();   
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it *= *itV++;
   return *this;
}

Vec& Vec::operator*=(double d) {
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it *= d;
   return *this;
}

Vec& Vec::operator/=(const Vec& V) {
   assert(n == V.n);
   std::vector<double>::const_iterator itV = V.data.begin();   
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it /= *itV++;
   return *this;
}

Vec& Vec::operator/=(double d) {
   for(std::vector<double>::iterator it = data.begin(); it != data.end(); ++it)
      *it /= d;
   return *this;
}

Vec Vec::operator+(const Vec& V) const {
   Vec tmp(*this);
   tmp += V;
   return tmp;
}

Vec Vec::operator+(double d) const {
   Vec tmp(*this);
   tmp += d;
   return tmp;
}

Vec Vec::operator-(const Vec& V) const {
   Vec tmp(*this);
   tmp -= V;
   return tmp;
}

Vec Vec::operator-(double d) const {
   Vec tmp(*this);
   tmp -= d;
   return tmp;
}

Vec Vec::operator*(const Vec& V) const {
   Vec tmp(*this);
   tmp *= V;
   return tmp;
}

Vec Vec::operator*(double d) const {
   Vec tmp(*this);
   tmp *= d;
   return tmp;
}

Vec Vec::operator/(const Vec& V) const {
   Vec tmp(*this);
   tmp /= V;
   return tmp;
}

Vec Vec::operator/(double d) const {
   Vec tmp(*this);
   tmp /= d;
   return tmp;
}
