#ifndef VEC_H
#define VEC_H

#include <vector>
#include "Mat2d.h"

class Mat2d;

class Vec {
private:
   int n;
   std::vector<double> data;

public:
   Vec();
   Vec(int n);
//   Vec(const Vec& V); // default copy-constructor sufficient
//   no destructor needed default destructor sufficient
   void resize(int n);
   int getN() const;
   Vec& fill(double value);
   const double& getElem(int i) const;
   double& getElem(int i);
   Vec& add(const Vec& V);
   bool equal(const Vec& V) const;
 
   // additional operators
   bool operator==(const Vec& V) const;
   bool operator!=(const Vec& V) const;
   // assignment operator - default sufficient
   //Vec& operator=(const Vec& V);
   Vec& operator=(double d);
   

   // operators
   const double& operator()(int i) const;
   double& operator()(int i);
   Vec& operator+=(const Vec& V);
   Vec& operator+=(double d);
   Vec& operator-=(const Vec& V);
   Vec& operator-=(double d);
   // elementwise multiplication
   Vec& operator*=(const Vec& V);
   Vec& operator*=(double d);
   Vec& operator/=(const Vec& V);
   Vec& operator/=(double d);
   Vec operator+(const Vec& V) const;
   Vec operator+(double d) const;
   Vec operator-(const Vec& V) const;
   Vec operator-(double d) const;
   Vec operator*(const Vec& V) const;
   Vec operator*(double d) const;
   Vec operator/(const Vec& V) const;
   Vec operator/(double d) const;

   friend Vec operator-(double d, const Vec& V);
   friend Vec operator/(double d, const Vec& V);

   friend Vec operator*(const Mat2d& M, const Vec& V);
};

#endif
