#ifndef BASICTESTS_H
#define BASICTESTS_H

#include "Mat2d.h"
#include "Vec.h"

bool testMat1() {
   const int m = 20;
   const int n = 10;
   Mat2d M1(m, n);
   Mat2d M2(m, n);
   
   // first test:
   M1.fill(1);
   M2.fill(2);
   
   if(!M1.add(M1).equal(M2))
      return false;

   return true;
}

bool testMat2() {
   const int m = 20;
   const int n = 10;
   Mat2d M1(m, n);
   // random test:
   // initialize
   for(int i = 0; i < m; ++i) {
      for(int j = 0; j < n; ++j) {
         M1.getElem(i, j) = rand();
      }
   }
   Mat2d M2(M1); 
   M1.add(M2);
   
   // test
   for(int i = 0; i < m; ++i) {
      for(int j = 0; j < n; ++j) {
         if(M1.getElem(i, j) != M2.getElem(i, j) * 2)
            return false;
      }
   }
   return true;
}

bool testVec1() {
   const int n = 100;
   Vec V1(n);
   Vec V2(n);

   V1.fill(1);
   V2.fill(2);
   
   if(!V1.add(V1).equal(V2))
      return false;

   return true;
}

bool testVec2() {
   const int n = 100;
   Vec V1(n);
   
   for(int i = 0; i < n; ++i)
      V1.getElem(i) = rand();

   Vec V2(V1);
   V1.add(V2);

   for(int i = 0; i < n; ++i) {
      if(V1.getElem(i) != V2.getElem(i) * 2) {
         return false;
      }
   }
   return true;
}

bool testMatVec() {
   bool result = true;
   if(!testMat1()) { 
      std::cerr << "Mat test1 failed" << std::endl;
      result = false;
   } else {
      std::cout << "Mat test1 passed" << std::endl;
   }
   
   if(!testMat2()) {
      std::cerr << "Mat test2 failed" << std::endl;
      result = false;
   } else {
      std::cout << "Mat test2 passed" << std::endl;
   }

   if(!testVec1()) {
      std::cerr << "Vec test1 failed" << std::endl;
      result = false;
   } else {
      std::cout << "Vec test1 passed" << std::endl;
   }

   if(!testVec2()) {
      std::cerr << "Vec test2 failed" << std::endl;
      result = false;
   } else {
      std::cout << "Vec test2 passed" << std::endl;
   }

   return result;
}

#endif
